package edu.travel.remote.upload;

import edu.travel.remote.upload.dto.EduFileBlobDTO;
import edu.travel.remote.upload.dto.EduFileDTO;
import edu.travel.rpc.RPCBaseResponse;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.Serializable;
import java.util.List;

@FeignClient(name = "upload",path = "/upload")
public interface UploadRemoteController {
    @PostMapping("/init")
    public RPCBaseResponse<String> initializeUpload();
    /**
     * 文件是否存在
     * @param md5
     * @return
     */
    @GetMapping("/exits/md5")
    RPCBaseResponse<Serializable> exitsFile(@RequestParam("md5") String md5);

    /**
     * 上传过哪些分片
     * @param md5
     * @return
     */
    @GetMapping("/exits/blob")
    RPCBaseResponse<List<Integer>> exitsBlob(@RequestParam("md5") String md5);

    /**
     * 上传大文件分片
     * @param file
     * @return
     */
    @PostMapping(value = "/uploadBlob",headers = {"content-type=multipart/form-data"})
    RPCBaseResponse uploadBigFile(EduFileBlobDTO dto,  MultipartFile file) throws IOException;

    /**
     * 合并大文件
     * @param eduFileDTO
     * @return
     */
    @PostMapping(value = "/mergeFile")
    RPCBaseResponse mergeFile(@RequestBody EduFileDTO eduFileDTO);

    /**
     * 上传小文件
     * @param file
     * @return
     */
    @PostMapping(value = "/upload",headers = {"content-type=multipart/form-data"})
    RPCBaseResponse uploadFile(EduFileDTO eduFile,MultipartFile file);
}
