package edu.travel.warehouse.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import edu.travel.adapter.service.country.CountryAdapter;
import edu.travel.adapter.service.tenant.SysRoleAdapter;
import edu.travel.adapter.service.tenant.TenantAdapter;
import edu.travel.dto.ShopWarehouseDto;
import edu.travel.dto.ShopWarehouseSaveDto;
import edu.travel.dto.ShopWarehouseStaffDto;
import edu.travel.remote.feign.mode.vo.tenant.EduTenantVo;
import edu.travel.remote.feign.mode.vo.tenant.SysRoleVo;
import edu.travel.rpc.RPCBaseResponse;
import edu.travel.service.SysServiceImpl;
import edu.travel.vo.BaseCountryServeVo;
import edu.travel.vo.ShopWarehouseStaffVo;
import edu.travel.vo.ShopWarehouseVo;
import edu.travel.warehouse.entity.ShopWarehouse;
import edu.travel.warehouse.entity.ShopWarehouseStaff;
import edu.travel.warehouse.mapper.ShopWarehouseMapper;
import edu.travel.warehouse.mapper.ShopWarehouseStaffMapper;
import edu.travel.warehouse.service.ShopWarehouseService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class ShopWarehouseServiceImpl extends SysServiceImpl<ShopWarehouseMapper, ShopWarehouse> implements ShopWarehouseService {
    @Autowired
    private CountryAdapter countryAdapter;
    @Autowired
    private TenantAdapter tenantAdapter;
    @Autowired
    private SysRoleAdapter sysRoleAdapter;
    @Autowired
    private ShopWarehouseMapper shopWarehouseMapper;
    @Autowired
    private ShopWarehouseStaffMapper shopWarehouseStaffMapper;
    /**
     * 树形结构查询（连表）
     */
    @Override
    public RPCBaseResponse<List<ShopWarehouseVo>> getWarehouseCountryForm(ShopWarehouseDto dto) {
        // 查询所有库房数据
        List<ShopWarehouse> shopWarehouseList = super.getListLink(new LambdaQueryWrapper<>());

        // 提取国家服务ID
        Set<String> countryServeIds = shopWarehouseList.stream()
                .map(ShopWarehouse::getCountryServeId)
                .collect(Collectors.toSet());

        // 查询国家信息
        Map<String, BaseCountryServeVo> baseCountryServeVoMap = fetchCountryData(countryServeIds);

        // 提取仓库 ID
        Set<Long> ids = shopWarehouseList.stream()
                .map(ShopWarehouse::getId)
                .collect(Collectors.toSet());

        // 获取用户和角色信息
        Map<Long, List<ShopWarehouseStaffVo>> staffMap = fetchWarehouseTenantRoles(ids);

        // 构建树形结构
        Map<Long, ShopWarehouseVo> warehouseVoMap = new HashMap<>();
        List<ShopWarehouseVo> tree = new ArrayList<>();

        for (ShopWarehouse shopWarehouse : shopWarehouseList) {
            ShopWarehouseVo shopWarehouseVo = mapToWarehouseVo(shopWarehouse, baseCountryServeVoMap, staffMap);
            warehouseVoMap.put(shopWarehouse.getId(), shopWarehouseVo);
            addToTree(tree, shopWarehouseVo, warehouseVoMap);
        }

        return new RPCBaseResponse<>(200, "SUCCESS", tree);
    }

    /**
     * 分页动态查询(连表)
     */
    @Override
    public RPCBaseResponse<IPage<ShopWarehouseVo>> getWarehouseCountryPageForm(ShopWarehouseDto dto) {
        // 创建分页对象
        Page<ShopWarehouseVo> shopWarehousePage = new Page<>(dto.getCurrentPage(), dto.getPageSize());

        // 查询库房数据并计算总记录数
        List<ShopWarehouseVo> warehouseList = shopWarehouseMapper.selectWarehouseWithCountry(
                dto.getWarehouseName(),
                dto.getStatus(),
                dto.getParentId(),
                dto.getPageSize(),
                (dto.getCurrentPage() - 1) * dto.getPageSize()
        );
        int total = shopWarehouseMapper.countWarehouseWithCountry(dto.getWarehouseName(), dto.getStatus(), dto.getParentId());

        // 提取国家服务ID
        Set<String> countryServeIds = warehouseList.stream()
                .map(ShopWarehouseVo::getCountryServeId)
                .collect(Collectors.toSet());

        // 查询国家信息
        Map<String, BaseCountryServeVo> baseCountryServeVoMap = fetchCountryData(countryServeIds);

        // 提取仓库 ID
        Set<Long> ids = warehouseList.stream()
                .map(ShopWarehouseVo::getId)
                .collect(Collectors.toSet());

        // 获取用户和角色信息
        Map<Long, List<ShopWarehouseStaffVo>> staffMap = fetchWarehouseTenantRoles(ids);

        // 将国家信息和用户角色信息放入对应的仓库列表项中
        for (ShopWarehouseVo warehouseVo : warehouseList) {
            if (baseCountryServeVoMap.containsKey(warehouseVo.getCountryServeId())) {
                warehouseVo.setMap(new HashMap<>()); // 初始化 map
                warehouseVo.getMap().put("countryServe", baseCountryServeVoMap.get(warehouseVo.getCountryServeId()));
            }
            if (staffMap.containsKey(warehouseVo.getId())) {
                warehouseVo.getMap().put("roles", staffMap.get(warehouseVo.getId()));
            }
        }

        // 封装返回
        IPage<ShopWarehouseVo> pageVoLink = new Page<>();
        pageVoLink.setRecords(warehouseList); // 设置转换后的记录列表
        pageVoLink.setTotal(total); // 设置总条数
        pageVoLink.setCurrent(shopWarehousePage.getCurrent()); // 设置当前页
        pageVoLink.setSize(shopWarehousePage.getSize()); // 设置每页大小

        return new RPCBaseResponse<>(200, "SUCCESS", pageVoLink);
    }

    /**
     * 提取国家数据
     */
    private Map<String, BaseCountryServeVo> fetchCountryData(Set<String> countryServeIds) {
        Map<String, BaseCountryServeVo> countryMap = new HashMap<>();
        if (!countryServeIds.isEmpty()) {
            for (String countryServeId : countryServeIds) {
                RPCBaseResponse<BaseCountryServeVo> response = countryAdapter.getFormId(countryServeId);
                if (response != null && response.getData() != null) {
                    countryMap.put(countryServeId, response.getData());
                }
            }
        }
        return countryMap;
    }

    /**
     * 提取仓库数据
     */
    private Map<Long, List<ShopWarehouseStaffVo>> fetchWarehouseTenantRoles(Set<Long> ids) {
        Map<Long, List<ShopWarehouseStaffVo>> staffMap = new HashMap<>();
        if (!ids.isEmpty()) {
            for (Long id : ids) {
                RPCBaseResponse<List<ShopWarehouseStaffVo>> response = getWarehouseTenantRole(String.valueOf(id));
                if (response != null && response.getData() != null) {
                    staffMap.put(id, response.getData());
                }
            }
        }
        return staffMap;
    }

    /**
     *生成map
     */
    private ShopWarehouseVo mapToWarehouseVo(ShopWarehouse shopWarehouse,
                                             Map<String, BaseCountryServeVo> countryMap,
                                             Map<Long, List<ShopWarehouseStaffVo>> staffMap) {
        ShopWarehouseVo shopWarehouseVo = new ShopWarehouseVo();
        shopWarehouseVo.setId(shopWarehouse.getId());
        shopWarehouseVo.setCountryServeId(shopWarehouse.getCountryServeId());
        shopWarehouseVo.setWarehouseName(shopWarehouse.getWarehouseName());
        shopWarehouseVo.setLongitude(shopWarehouse.getLongitude());
        shopWarehouseVo.setLatitude(shopWarehouse.getLatitude());
        shopWarehouseVo.setDetailedAddress(shopWarehouse.getDetailedAddress());
        shopWarehouseVo.setStatus(shopWarehouse.getStatus());
        shopWarehouseVo.setProject(shopWarehouse.getProject());
        shopWarehouseVo.setParentId(shopWarehouse.getParentId());
        shopWarehouseVo.setDeleteFlag(shopWarehouse.getDeleteFlag());
        shopWarehouseVo.setMap(new HashMap<>());

        if (countryMap.containsKey(shopWarehouse.getCountryServeId())) {
            shopWarehouseVo.getMap().put("countryServe", countryMap.get(shopWarehouse.getCountryServeId()));
        }
        if (staffMap.containsKey(shopWarehouse.getId())) {
            shopWarehouseVo.getMap().put("roles", staffMap.get(shopWarehouse.getId()));
        }
        return shopWarehouseVo;
    }

    /**
     * 构建树
     */
    private void addToTree(List<ShopWarehouseVo> tree, ShopWarehouseVo warehouseVo, Map<Long, ShopWarehouseVo> warehouseVoMap) {
        if (warehouseVo.getParentId() == null || warehouseVo.getParentId() == 0) {
            tree.add(warehouseVo);
        } else {
            ShopWarehouseVo parentVo = warehouseVoMap.get(warehouseVo.getParentId());
            if (parentVo != null) {
                if (parentVo.getChildren() == null) {
                    parentVo.setChildren(new ArrayList<>());
                }
                parentVo.getChildren().add(warehouseVo);
            }
        }
    }


    /**
     * 库房关联人员(角色）查询，通过库房id，查询用户以及角色
     */
    @Override
    public RPCBaseResponse<List<ShopWarehouseStaffVo>> getWarehouseTenantRole(String id) {
        // 查询所有库房扩展数据
        LambdaQueryWrapper<ShopWarehouseStaff> queryWrapper = new LambdaQueryWrapper<>();
        List<ShopWarehouseStaff> shopWarehouseStaffList = shopWarehouseStaffMapper.selectList(queryWrapper);

        // 提取用户id和角色id
        List<Long> userIdList = shopWarehouseStaffList.stream()
                .filter(staff -> staff.getWarehouseId().toString().equals(id))
                .map(ShopWarehouseStaff::getUserId)
                .collect(Collectors.toList());

        List<Long> roleIdList = shopWarehouseStaffList.stream()
                .filter(staff -> staff.getWarehouseId().toString().equals(id))
                .map(ShopWarehouseStaff::getRoleId)
                .collect(Collectors.toList());

        // 创建 map 以便通过 userId 和 roleId 查询出用户和角色信息
        Map<Long, EduTenantVo> tenantMap = new HashMap<>();
        if (!userIdList.isEmpty()) {
            for (Long userId : userIdList) {
                RPCBaseResponse<EduTenantVo> tenantVoRPCBaseResponse = tenantAdapter.getTenantById(userId);
                if (tenantVoRPCBaseResponse != null && tenantVoRPCBaseResponse.getData() != null) {
                    tenantMap.put(userId, tenantVoRPCBaseResponse.getData());
                }
            }
        }

        Map<Long, SysRoleVo> roleMap = new HashMap<>();
        if (!roleIdList.isEmpty()) {
            for (Long roleId : roleIdList) {
                RPCBaseResponse<SysRoleVo> roleVoRPCBaseResponse = sysRoleAdapter.getFormId(String.valueOf(roleId));
                if (roleVoRPCBaseResponse != null && roleVoRPCBaseResponse.getData() != null) {
                    roleMap.put(roleId, roleVoRPCBaseResponse.getData());
                }

            }
        }

        // 组合结果
        List<ShopWarehouseStaffVo> resultList = new ArrayList<>();
        for (ShopWarehouseStaff staff : shopWarehouseStaffList) {
            if (staff.getWarehouseId().toString().equals(id)) {
                Long userId = staff.getUserId();
                Long roleId = staff.getRoleId();

                EduTenantVo user = tenantMap.get(userId);
                SysRoleVo role = roleMap.get(roleId);

                // 封装用户和角色信息
                ShopWarehouseStaffVo result = new ShopWarehouseStaffVo();
                result.setUserId(userId);
                result.setRoleId(roleId);
                result.setWarehouseId(staff.getWarehouseId());

                if (user != null) {
                    result.setTenantSurname(user.getTenantSurname());
                    result.setTenantName(user.getTenantName());
                    //拼接全名
                    result.setTenantNameFull(user.getTenantSurname() + " " + user.getTenantName());
                }

                if (role != null) {
                    result.setRoleId(roleId);
                    result.setName(role.getName());
                }

                resultList.add(result);
            }
        }

        // 返回成功响应
        return new RPCBaseResponse<>(200, "SUCCESS", resultList);
    }

    /**
     * 连表新增
     */
    @Override
    @Transactional // 确保此方法在事务中执行
    public RPCBaseResponse<ShopWarehouseVo> saveAllForm(ShopWarehouseSaveDto entity) {
        // 创建并设置仓库信息
        ShopWarehouse warehouse = new ShopWarehouse();
        copyWarehouseProperties(entity, warehouse);

        // 插入仓库信息
        shopWarehouseMapper.insert(warehouse);

        // 保存员工角色信息
        saveStaffRoles(entity.getStaffList(), warehouse.getId());

        // 构建返回对象
        return buildWarehouseResponse(warehouse);
    }

    // 从 DTO 复制属性到实体
    private void copyWarehouseProperties(ShopWarehouseSaveDto source, ShopWarehouse target) {
        target.setParentId(source.getParentId());
        target.setWarehouseName(source.getWarehouseName());
        target.setLongitude(source.getLongitude());
        target.setLatitude(source.getLatitude());
        target.setCountryServeId(source.getCountryServeId());
        target.setDetailedAddress(source.getDetailedAddress());
        target.setStatus(source.getStatus());
        // 其他需要设置的字段...
    }

    // 保存员工角色
    private void saveStaffRoles(List<ShopWarehouseStaffDto> staffList, Long warehouseId) {
        if (staffList != null && !staffList.isEmpty()) {
            for (ShopWarehouseStaffDto staffDto : staffList) {
                ShopWarehouseStaff staff = new ShopWarehouseStaff();
                staff.setWarehouseId(warehouseId);
                staff.setUserId(staffDto.getUserId());
                staff.setRoleId(staffDto.getRoleId());
                shopWarehouseStaffMapper.insert(staff);
            }
        }
    }

    // 构建返回对象
    private RPCBaseResponse<ShopWarehouseVo> buildWarehouseResponse(ShopWarehouse warehouse) {
        ShopWarehouseVo warehouseVo = new ShopWarehouseVo();
        warehouseVo.setId(warehouse.getId());
        warehouseVo.setWarehouseName(warehouse.getWarehouseName());
        // 其他字段映射...
        return new RPCBaseResponse<>(200, "SUCCESS", warehouseVo);
    }



    /**
     * 更新
     */
    @Override
    @Transactional // 确保此方法在事务中执行
    public RPCBaseResponse<ShopWarehouseVo> updateWarehouse(ShopWarehouseSaveDto entity) {
        // 更新仓库信息
        ShopWarehouse warehouse = new ShopWarehouse();
        warehouse.setId(entity.getId());
        copyWarehouseProperties(entity, warehouse);
        shopWarehouseMapper.updateById(warehouse);

        // 更新员工角色信息
        updateStaffRoles(entity.getStaffList(), entity.getId());

        // 构建返回对象
        return buildWarehouseResponse(warehouse);
    }

    // 更新员工角色
    private void updateStaffRoles(List<ShopWarehouseStaffDto> staffList, Long warehouseId) {
        // 删除原有的员工记录
        shopWarehouseStaffMapper.deleteByWarehouseId(warehouseId);

        // 插入新的员工角色信息
        saveStaffRoles(staffList, warehouseId);
    }


}
